import React from "react";
import {
  PRE_GAME,
  PRE_ROUND,
  IN_ROUND,
  POST_ROUND,
  POST_GAME,
  ERROR,
} from "../../domain/GameState";
import PreGame from '../screens/PreGame';
import PreRound from '../screens/PreRound';
import GamePanel from "../screens/GamePanel";
import RoundSummary from '../screens/RoundSummary';
import PlayerScores from "../screens/PlayerScores";
import { gameIdStore, gameStateStore } from "../../domain/store";
import useObservable from "../../hooks/useObservable";
import { useEffect } from "react";

const extractAndRemoveSearchParam = param => {
  const u = new URL(window.location.href);
  const extracted = u.searchParams.get(param);
  u.searchParams.delete(param);
  window.history.replaceState({}, document.title, u.href);
  return extracted;
}

const componentMap = {
  [PRE_GAME]: PreGame,
  [PRE_ROUND]: PreRound,
  [IN_ROUND]: GamePanel,
  [POST_ROUND]: RoundSummary,
  [POST_GAME]: PlayerScores,
  [ERROR]: () => <p>Application encountered unrecoverable error, please refresh the page.</p>,
}

const Game = () => {
  const gameState = useObservable(gameStateStore);

  useEffect(() => {
    const joinCode = extractAndRemoveSearchParam("join");
    if (joinCode) {
      gameIdStore.set(joinCode);
      gameStateStore.set(PRE_ROUND);
    }
  }, [])

  useEffect(() => {
    const summaryCode = extractAndRemoveSearchParam("summary");
    if (summaryCode) {
      gameIdStore.set(summaryCode);
      gameStateStore.set(POST_GAME);
    }
  }, [])

  const Screen = componentMap[gameState];

  return <Screen />
}

export default Game;
